@@ -6,6 +6,7 @@ import Offcanvas from 'bootstrap/js/src/offcanvas'
 import Tab from 'bootstrap/js/src/tab'
 import Toast from 'bootstrap/js/src/toast'
 import Tags from 'bootstrap5-tags/tags'
+import semverCompare from 'semver/functions/compare'
 
 import {
   AUTO_LAUNCH_PERMISSIONS,
@@ -13,6 +14,7 @@ import {
   checkNativeStatus,
   getConfig,
   getIcon,
+  getLatestAmoVersion,
   isAutoRuntimeInstallSupported,
   isProtocolSchemePermitted,
   launchSite,
@@ -20,6 +22,7 @@ import {
   obtainSiteList,
   PREF_AUTO_LAUNCH_EXCLUSION,
   PREF_DEFAULT_PROFILE_TEMPLATE,
+  PREF_DEFAULT_TAB,
   PREF_DISPLAY_PAGE_ACTION,
   PREF_ENABLE_AUTO_LAUNCH,
   PREF_LAUNCH_CURRENT_URL,
@@ -34,25 +37,111 @@ import { knownCategories } from './categories'
 
 // Display install/update page when clicked on browser action and the native program is not correctly installed
 async function handleNativeStatus () {
-  switch (await checkNativeStatus()) {
+  const nativeStatus = await checkNativeStatus()
+  switch (nativeStatus.status) {
     case 'install':
       await browser.tabs.create({ url: browser.runtime.getURL('setup/install.html') })
       window.close()
       break
     case 'update-major':
-      await browser.tabs.create({ url: browser.runtime.getURL('setup/update.html') })
-      window.close()
-      break
     case 'update-minor': {
-      const outdatedBox = document.getElementById('extension-outdated-box')
-      document.getElementById('extension-outdated-update').setAttribute('href', browser.runtime.getURL('setup/update.html'))
-      document.getElementById('extension-outdated-close').addEventListener('click', () => outdatedBox.classList.add('d-none'))
-      outdatedBox.classList.remove('d-none')
+      const comparedVersions = semverCompare(nativeStatus.extension, nativeStatus.native)
+
+      const outdatedBox = document.getElementById('outdated-box')
+      const outdatedBoxNotice = document.getElementById('outdated-box-notice')
+      const outdatedBoxPending = document.getElementById('outdated-box-pending')
+      const outdatedBoxIncompatible = document.getElementById('outdated-box-incompatible')
+      const outdatedBoxLink = document.getElementById('outdated-box-link')
+      const outdatedBoxClose = document.getElementById('outdated-box-close')
+
+      let displayNotice = false
+
+      // Native program is outdated
+      if (comparedVersions > 0) {
+        // Display notice for outdated native program
+        outdatedBoxNotice.innerText = await getMessage('managePageFooterOutdatedNative')
+        outdatedBoxLink.innerText = await getMessage('managePageFooterOutdatedNativeUpdate')
+
+        // We assume the native program is always available
+        outdatedBoxPending.classList.add('d-none')
+        outdatedBoxLink.classList.remove('d-none')
+
+        // Always display the notice for outdated native program
+        displayNotice = true
+      }
+
+      // Extension is outdated
+      if (comparedVersions < 0) {
+        // Display notice for outdated extension
+        outdatedBoxNotice.innerText = await getMessage('managePageFooterOutdatedExtension')
+        outdatedBoxLink.innerText = await getMessage('managePageFooterOutdatedExtensionUpdate')
+
+        // Check the latest available extension version
+        const latestExtension = await getLatestAmoVersion()
+
+        if (!latestExtension || semverCompare(nativeStatus.native, latestExtension) <= 0) {
+          // The latest version is available
+          outdatedBoxPending.classList.add('d-none')
+          outdatedBoxLink.classList.remove('d-none')
+
+          // Always display the notice for outdated extension with update available
+          displayNotice = true
+        } else {
+          // The latest version is not yet available
+          outdatedBoxPending.classList.remove('d-none')
+          outdatedBoxLink.classList.add('d-none')
+
+          // Do not display the notice for outdated extension without update available
+          displayNotice = false
+        }
+      }
+
+      // Always display incompatible notice if major versions mismatch
+      if (nativeStatus.status === 'update-major') {
+        outdatedBoxIncompatible.classList.remove('d-none')
+        displayNotice = true
+      } else {
+        outdatedBoxIncompatible.classList.add('d-none')
+      }
+
+      // Open the update page on link click
+      outdatedBoxLink.setAttribute('href', browser.runtime.getURL('setup/update.html'))
+
+      // Close the notice on close click
+      outdatedBoxClose.addEventListener('click', () => outdatedBox.classList.add('d-none'))
+
+      // Display the notice if necessary
+      if (displayNotice) outdatedBox.classList.remove('d-none')
+      else outdatedBox.classList.add('d-none')
     }
       break
   }
 }
 
+// Set a default tab based on the setting
+async function setDefaultTab () {
+  const settingsDefaultTab = (await browser.storage.local.get([PREF_DEFAULT_TAB]))[PREF_DEFAULT_TAB] ?? 'site-grid'
+
+  switch (settingsDefaultTab) {
+    case 'site-grid':
+      document.getElementById('grid-tab').classList.add('active')
+      document.getElementById('grid-pane').classList.add('show', 'active')
+      break
+    case 'site-list':
+      document.getElementById('sites-tab').classList.add('active')
+      document.getElementById('sites-pane').classList.add('show', 'active')
+      break
+    case 'profiles':
+      document.getElementById('profiles-tab').classList.add('active')
+      document.getElementById('profiles-pane').classList.add('show', 'active')
+      break
+    case 'settings':
+      document.getElementById('settings-tab').classList.add('active')
+      document.getElementById('settings-pane').classList.add('show', 'active')
+      break
+  }
+}
+
 // Fill the site list
 async function createSiteList () {
   const siteInstallButton = document.getElementById('site-install-button')
@@ -75,20 +164,35 @@ async function createSiteList () {
 
   // Get the list elements
   const listElement = document.getElementById('sites-list')
+  const gridContainer = document.getElementById('grid-list')
   const templateElement = document.getElementById('sites-list-template')
+  const gridTemplateElement = document.getElementById('grid-list-template')
   const loadingElement = document.getElementById('sites-list-loading')
+  const gridLoadingElement = document.getElementById('grid-list-loading')
   const emptyElement = document.getElementById('sites-list-empty')
+  const gridEmptyElement = document.getElementById('grid-list-empty')
 
   loadingElement.classList.add('d-none')
-  if (!sites.length) emptyElement.classList.remove('d-none')
+  gridLoadingElement.classList.add('d-none')
+
+  if (!sites.length) {
+    emptyElement.classList.remove('d-none')
+    gridEmptyElement.classList.remove('d-none')
+  }
 
   // Create a list element for every instance with handlers for launching and editing
   for (const site of sites) {
+    // Create list view item
     const siteElement = templateElement.content.firstElementChild.cloneNode(true)
+
+    // Create grid view item
+    const gridItem = gridTemplateElement.content.firstElementChild.cloneNode(true)
+
     const siteName = sanitizeString(site.config.name || site.manifest.name || site.manifest.short_name) || new URL(site.manifest.scope).host
     const siteDescription = sanitizeString(site.config.description || site.manifest.description) || ''
     const siteIcon = site.config.icon_url || getIcon(buildIconList(site.manifest.icons), 64)
 
+    // Setup list view item
     const letterElement = siteElement.querySelector('#sites-list-template-letter')
     if (siteIcon) letterElement.classList.add('d-none')
     letterElement.setAttribute('data-letter', siteName[0])
@@ -104,6 +208,71 @@ async function createSiteList () {
       iconElement.classList.add('d-none')
     }
 
+    // Setup grid view item
+    const gridLetterElement = gridItem.querySelector('#grid-list-template-letter')
+    if (siteIcon) gridLetterElement.classList.add('d-none')
+    gridLetterElement.setAttribute('data-letter', siteName[0])
+    gridLetterElement.removeAttribute('id')
+
+    const gridIconElement = gridItem.querySelector('#grid-list-template-icon')
+    if (!siteIcon) gridIconElement.classList.add('d-none')
+    gridIconElement.src = siteIcon
+    gridIconElement.setAttribute('alt', await getMessage('managePageAppListIcon'))
+    gridIconElement.removeAttribute('id')
+    gridIconElement.onerror = () => {
+      gridLetterElement.classList.remove('d-none')
+      gridIconElement.classList.add('d-none')
+    }
+
+    // Handle grid item clicks to show/hide buttons
+    const buttonsPopup = gridItem.querySelector('.grid-item-buttons')
+    gridItem.addEventListener('click', (event) => {
+      // Don't show popup if clicking on a button
+      if (event.target.closest('.grid-item-buttons')) {
+        return
+      }
+
+      // Remove active class from all other items
+      document.querySelectorAll('.grid-item').forEach(item => {
+        if (item !== gridItem) {
+          item.classList.remove('active')
+        }
+      })
+
+      // Toggle active class on clicked item
+      gridItem.classList.toggle('active')
+
+      if (gridItem.classList.contains('active')) {
+        // Position the popup at click coordinates
+        buttonsPopup.style.top = `${event.clientY}px`
+        buttonsPopup.style.left = `${event.clientX}px`
+
+        // Adjust position if popup would go off screen
+        const rect = buttonsPopup.getBoundingClientRect()
+        const viewportWidth = document.documentElement.clientWidth
+        const viewportHeight = document.documentElement.clientHeight
+
+        if (rect.right > viewportWidth) {
+          buttonsPopup.style.left = `${event.clientX - rect.width}px`
+        }
+        if (rect.bottom > viewportHeight) {
+          buttonsPopup.style.top = `${event.clientY - rect.height}px`
+        }
+      }
+
+      event.stopPropagation()
+    })
+
+    // Close popup when clicking outside
+    document.addEventListener('click', (event) => {
+      if (!event.target.closest('.grid-item')) {
+        document.querySelectorAll('.grid-item').forEach(item => {
+          item.classList.remove('active')
+        })
+      }
+    })
+
+    // Set titles and descriptions
     const titleElement = siteElement.querySelector('#sites-list-template-title')
     titleElement.innerText = siteName
     titleElement.removeAttribute('id')
@@ -112,16 +281,32 @@ async function createSiteList () {
     descriptionElement.innerText = siteDescription
     descriptionElement.removeAttribute('id')
 
+    const gridTitleElement = gridItem.querySelector('#grid-list-template-title')
+    gridTitleElement.innerText = siteName
+    gridTitleElement.removeAttribute('id')
+
+    // Setup launch buttons
     const launchElement = siteElement.querySelector('#sites-list-template-launch')
+    const gridLaunchElement = gridItem.querySelector('#grid-list-template-launch')
     const launchElementTooltip = await getMessage('managePageAppListLaunch')
+
     launchElement.addEventListener('click', () => { launchSite(site) })
+    gridLaunchElement.addEventListener('click', () => { launchSite(site) })
+
     launchElement.setAttribute('title', launchElementTooltip)
     launchElement.setAttribute('aria-label', launchElementTooltip)
     launchElement.removeAttribute('id')
 
+    gridLaunchElement.setAttribute('title', launchElementTooltip)
+    gridLaunchElement.setAttribute('aria-label', launchElementTooltip)
+    gridLaunchElement.removeAttribute('id')
+
+    // Setup edit buttons
     const editElement = siteElement.querySelector('#sites-list-template-edit')
+    const gridEditElement = gridItem.querySelector('#grid-list-template-edit')
     const editElementTooltip = await getMessage('managePageAppListEdit')
-    editElement.addEventListener('click', async (event) => {
+
+    const editHandler = async (event) => {
       const form = document.getElementById('web-app-form')
       const submit = document.getElementById('web-app-submit')
 
@@ -242,31 +427,36 @@ async function createSiteList () {
       // Validate start URL input
       const startUrlValidation = async function () {
         const invalidLabel = document.getElementById('web-app-start-url-invalid')
+        const warningLabel = document.getElementById('web-app-start-url-warning')
 
         // Empty URL defaults to manifest start URL
         if (!this.value) {
           this.setCustomValidity('')
+          this.classList.remove('is-warning')
           return
         }
 
         // Start URL needs to be a valid URL
         if (this.validity.typeMismatch) {
+          this.classList.remove('is-warning')
           this.setCustomValidity(await getMessage('webAppValidationStartURLInvalid'))
           invalidLabel.innerText = this.validationMessage
           return
         }
 
-        // Start URL needs to be within the scope
+        // Start URL should be within the scope
         const startUrl = new URL(this.value)
         const scope = new URL(site.manifest.scope)
         if (startUrl.origin !== scope.origin || !startUrl.pathname.startsWith(scope.pathname)) {
-          this.setCustomValidity(`${await getMessage('webAppValidationStartURLScope')} ${scope}`)
-          invalidLabel.innerText = this.validationMessage
+          this.setCustomValidity('')
+          warningLabel.innerText = `${await getMessage('webAppValidationStartURLScope')} ${scope}`
+          this.classList.add('is-warning')
           return
         }
 
         // All checks passed
         this.setCustomValidity('')
+        this.classList.remove('is-warning')
       }
 
       const startUrlInput = document.getElementById('web-app-start-url')
@@ -374,14 +564,25 @@ async function createSiteList () {
       // Show offcanvas element
       Offcanvas.getOrCreateInstance(document.getElementById('site-edit-offcanvas')).show()
       event.preventDefault()
-    })
+    }
+
+    editElement.addEventListener('click', editHandler)
+    gridEditElement.addEventListener('click', editHandler)
+
     editElement.setAttribute('title', editElementTooltip)
     editElement.setAttribute('aria-label', editElementTooltip)
     editElement.removeAttribute('id')
 
+    gridEditElement.setAttribute('title', editElementTooltip)
+    gridEditElement.setAttribute('aria-label', editElementTooltip)
+    gridEditElement.removeAttribute('id')
+
+    // Setup remove buttons
     const removeElement = siteElement.querySelector('#sites-list-template-remove')
+    const gridRemoveElement = gridItem.querySelector('#grid-list-template-remove')
     const removeElementTooltip = await getMessage('managePageAppListRemove')
-    removeElement.addEventListener('click', () => {
+
+    const removeHandler = () => {
       const lastSiteInProfile = profiles[site.profile].sites.length <= 1
 
       document.getElementById('site-remove-button').onclick = async function () {
@@ -428,12 +629,21 @@ async function createSiteList () {
       }
 
       Modal.getOrCreateInstance(document.getElementById('site-remove-modal')).show()
-    })
+    }
+
+    removeElement.addEventListener('click', removeHandler)
+    gridRemoveElement.addEventListener('click', removeHandler)
+
     removeElement.setAttribute('title', removeElementTooltip)
     removeElement.setAttribute('aria-label', removeElementTooltip)
     removeElement.removeAttribute('id')
 
+    gridRemoveElement.setAttribute('title', removeElementTooltip)
+    gridRemoveElement.setAttribute('aria-label', removeElementTooltip)
+    gridRemoveElement.removeAttribute('id')
+
     listElement.insertBefore(siteElement, templateElement)
+    gridContainer.insertBefore(gridItem, gridTemplateElement)
   }
 }
 
@@ -684,17 +894,25 @@ async function createProfileList () {
 
 // Handle site and profile search
 async function handleSearch () {
-  const searchHandler = function (listElement) {
+  const searchHandler = function (listElement, gridElement) {
     document.getElementById('search-box').classList.remove('invisible')
 
     document.getElementById('search-input').oninput = function () {
+      const searchQuery = sanitizeString(this.value.toLowerCase())
+
       for (const item of document.getElementById(listElement).children) {
         const itemName = sanitizeString(item.querySelector('.list-group-item-name')?.innerText.toLowerCase())
-        const searchQuery = sanitizeString(this.value.toLowerCase())
-
         if (!itemName) continue
         item.classList.toggle('d-none', itemName.indexOf(searchQuery) === -1)
       }
+
+      if (gridElement) {
+        for (const item of document.getElementById(gridElement).children) {
+          const itemName = sanitizeString(item.querySelector('.list-group-item-name')?.innerText.toLowerCase())
+          if (!itemName) continue
+          item.classList.toggle('d-none', itemName.indexOf(searchQuery) === -1)
+        }
+      }
     }
   }
 
@@ -702,17 +920,18 @@ async function handleSearch () {
     document.getElementById('search-box').classList.add('invisible')
   }
 
+  document.getElementById('grid-tab').addEventListener('click', () => searchHandler('grid-list'))
   document.getElementById('sites-tab').addEventListener('click', () => searchHandler('sites-list'))
   document.getElementById('profiles-tab').addEventListener('click', () => searchHandler('profiles-list'))
   document.getElementById('settings-tab').addEventListener('click', () => searchHide())
 
-  searchHandler('sites-list')
+  searchHandler('grid-list')
 }
 
 // Handle extension settings
 async function handleSettings (hasChanged = false) {
   // Get settings from local storage and media query
-  const settings = await browser.storage.local.get([PREF_DISPLAY_PAGE_ACTION, PREF_LAUNCH_CURRENT_URL, PREF_SHOW_UPDATE_POPUP, PREF_ENABLE_AUTO_LAUNCH, PREF_DEFAULT_PROFILE_TEMPLATE, PREF_AUTO_LAUNCH_EXCLUSION])
+  const settings = await browser.storage.local.get([PREF_DISPLAY_PAGE_ACTION, PREF_LAUNCH_CURRENT_URL, PREF_SHOW_UPDATE_POPUP, PREF_ENABLE_AUTO_LAUNCH, PREF_DEFAULT_PROFILE_TEMPLATE, PREF_AUTO_LAUNCH_EXCLUSION, PREF_DEFAULT_TAB])
   const settingsDisplayPageAction = settings[PREF_DISPLAY_PAGE_ACTION] ? settings[PREF_DISPLAY_PAGE_ACTION] : 'valid'
   const settingsLaunchCurrentUrl = settings[PREF_LAUNCH_CURRENT_URL] !== undefined ? settings[PREF_LAUNCH_CURRENT_URL] : true
   const settingsShowUpdatePopup = settings[PREF_SHOW_UPDATE_POPUP] !== undefined ? settings[PREF_SHOW_UPDATE_POPUP] : true
@@ -720,6 +939,7 @@ async function handleSettings (hasChanged = false) {
   const settingsEnableDarkMode = window.matchMedia && window.matchMedia('(prefers-color-scheme: dark)').matches
   const settingsDefaultProfileTemplate = settings[PREF_DEFAULT_PROFILE_TEMPLATE] || null
   const settingsAutoLaunchExclusion = settings[PREF_AUTO_LAUNCH_EXCLUSION] || null
+  const settingsDefaultTab = settings[PREF_DEFAULT_TAB] ?? 'site-grid'
 
   // Set settings input values
   document.getElementById('settings-display-page-action').querySelector(`#settings-display-page-action-${settingsDisplayPageAction}`).checked = true
@@ -729,6 +949,7 @@ async function handleSettings (hasChanged = false) {
   document.getElementById('settings-enable-dark-mode').checked = settingsEnableDarkMode
   document.getElementById('settings-default-profile-template').value = settingsDefaultProfileTemplate
   document.getElementById('settings-auto-launch-exclusion').value = settingsAutoLaunchExclusion
+  document.getElementById('settings-default-tab').value = settingsDefaultTab
 
   // Do not re-register listeners
   if (hasChanged) return
@@ -791,6 +1012,11 @@ async function handleSettings (hasChanged = false) {
     await browser.storage.local.set({ [PREF_AUTO_LAUNCH_EXCLUSION]: this.value || null })
   })
 
+  // Handle default tab selection
+  document.getElementById('settings-default-tab').addEventListener('change', async function () {
+    await browser.storage.local.set({ [PREF_DEFAULT_TAB]: this.value })
+  })
+
   // Handle language selection
   const languageElement = document.getElementById('settings-language')
   const languageNames = new Intl.DisplayNames(['en'], { type: 'language', languageDisplay: 'standard' })
@@ -989,6 +1215,7 @@ handleNativeStatus()
 for (const element of document.querySelectorAll('.form-select-tags')) { element.tagsInstance = new Tags(element) }
 Tab.getOrCreateInstance(document.getElementById('card-navigation'))
 setPopupSize()
+setDefaultTab()
 createSiteList()
 createProfileList()
 handleSearch()
