@@ -3,10 +3,10 @@ use std::io::Result as IoResult;
 use std::path::{Path, PathBuf};
 use std::process::{Child, Command};
 
-use anyhow::{anyhow, Context, Result};
+use anyhow::{Context, Result, anyhow};
 use cfg_if::cfg_if;
 use configparser::ini::Ini;
-use fs_extra::dir::{copy, CopyOptions};
+use fs_extra::dir::{CopyOptions, copy};
 use log::{info, warn};
 use tempfile::{NamedTempFile, TempDir};
 
@@ -57,7 +57,7 @@ cfg_if! {
         use std::ffi::c_void;
         use std::mem;
 
-        use windows::Win32::Foundation::{BOOL, HANDLE};
+        use windows::core::BOOL;
         use windows::Win32::System::JobObjects::{
             IsProcessInJob,
             JobObjectExtendedLimitInformation,
@@ -74,7 +74,7 @@ cfg_if! {
         /// the limit breakaway ok flag is set.
         fn allows_breakaway_from_job() -> Result<bool> {
             let mut process_in_job: BOOL = BOOL(0);
-            unsafe { IsProcessInJob(GetCurrentProcess(), HANDLE(std::ptr::null_mut()), &mut process_in_job)? }
+            unsafe { IsProcessInJob(GetCurrentProcess(), None, &mut process_in_job)? }
 
             if process_in_job.0 == 0 {
                 return Ok(true);
@@ -84,7 +84,7 @@ cfg_if! {
 
             unsafe {
                 QueryInformationJobObject(
-                    HANDLE(std::ptr::null_mut()),
+                    None,
                     JobObjectExtendedLimitInformation,
                     &mut info as *mut _ as *mut c_void,
                     mem::size_of_val(&info) as u32,
@@ -139,6 +139,8 @@ fn get_download_url() -> &'static str {
             concatcp!(BASE_DOWNLOAD_URL, "linux")
         } else if #[cfg(all(platform_linux, target_arch = "x86_64"))] {
             concatcp!(BASE_DOWNLOAD_URL, "linux64")
+        } else if #[cfg(all(platform_linux, target_arch = "aarch64"))] {
+            concatcp!(BASE_DOWNLOAD_URL, "linux64-aarch64")
         } else if #[cfg(platform_macos)] {
             concatcp!(BASE_DOWNLOAD_URL, "osx")
         } else {
@@ -240,7 +242,7 @@ impl Runtime {
         const COPY_ERROR: &str = "Failed to copy the runtime";
         const CLEANUP_ERROR: &str = "Failed to clean up the runtime";
 
-        #[cfg(feature = "linked-runtime")]
+        #[cfg(platform_linux)]
         {
             use crate::storage::Storage;
 
@@ -259,8 +261,10 @@ impl Runtime {
         warn!("Firefox is licensed under the Mozilla Public License 2.0");
         warn!("Firefox is a trademark of the Mozilla Foundation in the U.S. and other countries");
         warn!("This project is not affiliated with the Mozilla Foundation in any way");
-        warn!("By using this project you also agree to the Firefox Privacy Notice: https://www.mozilla.org/privacy/firefox/");
-        warn!("Check the Firefox website for more details: https://www.mozilla.org/firefox/");
+        warn!("By using the runtime you agree to the Firefox Terms of Use and Privacy Notice");
+        warn!("Firefox Terms of Use: https://www.mozilla.org/about/legal/terms/firefox/");
+        warn!("Firefox Privacy Notice: https://www.mozilla.org/privacy/firefox/");
+        warn!("Firefox Website: https://www.mozilla.org/firefox/");
 
         info!("Downloading the runtime archive");
         let mut archive = NamedTempFile::new().context(TEMP_FILE_ERROR)?;
@@ -288,29 +292,35 @@ impl Runtime {
                 let _7zip = _7Zip::new()?;
                 let success = _7zip.run(vec!["x", &archive, &format!("-o{}", &extracted)]).context(EXTRACT_ERROR)?.success();
                 if !success { bail!(EXTRACT_ERROR) }
+
                 source.push("core");
+
             } else if #[cfg(platform_linux)] {
                 use std::fs::File;
-                use bzip2::read::BzDecoder;
+                use xz2::read::XzDecoder;
                 use tar::Archive;
 
-                let mut compressed = Archive::new(BzDecoder::new(File::open(&archive)?));
+                let file = File::open(&archive).context(EXTRACT_ERROR)?;
+                let mut compressed = Archive::new(XzDecoder::new(file));
                 compressed.unpack(&extracted).context(EXTRACT_ERROR)?;
+
                 source.push("firefox");
+
             } else if #[cfg(platform_macos)] {
                 use dmg::Attach;
 
                 let info = Attach::new(&archive).with().context(EXTRACT_ERROR)?;
-                let mut options = CopyOptions::new();
                 let mut mount_point = info.mount_point.clone();
 
                 mount_point.push("Firefox.app");
                 source.push("Firefox.app");
 
+                let mut options = CopyOptions::new();
                 options.content_only = true;
                 copy(&mount_point, &source, &options)?;
 
                 source.pop();
+
             } else {
                 panic!("{}", UNSUPPORTED_PLATFORM_ERROR);
             }
@@ -331,7 +341,7 @@ impl Runtime {
         Ok(())
     }
 
-    #[cfg(feature = "linked-runtime")]
+    #[cfg(all(platform_linux, not(feature = "immutable-runtime")))]
     pub fn link(&self) -> Result<()> {
         use std::fs::{copy, create_dir_all};
         use std::os::unix::fs::symlink;
