@@ -45,7 +45,7 @@ macro_rules! deserialize_unit_struct {
         impl<'de> Deserialize<'de> for $msg {
             fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
             where
-                D: serde::de::Deserializer<'de>,
+                D: Deserializer<'de>,
             {
                 struct Visitor;
 
@@ -77,7 +77,7 @@ const fn default_as_true() -> bool {
     true
 }
 
-/// Supports "double option" pattern for update requests.
+/// Supports the "double option" pattern for update requests.
 ///
 /// - Parses missing field as `None`.
 /// - Parses field with value `None` as `Some(None)`.
@@ -143,27 +143,24 @@ pub struct SetConfig(pub Config);
 ///
 /// - Windows: All (x86, x64, ARM64)
 /// - MacOS: All (x64, ARM64)
-/// - Linux: x86, x64
+/// - Linux: x86, x64, ARM64
 ///
 /// # Parameters
 ///
-/// None.
+/// See [fields](#fields).
 ///
 /// # Returns
 ///
 /// [`ConnectorResponse::RuntimeInstalled`] - No data.
 ///
-
-#[cfg(feature = "linked-runtime")]
-#[derive(Deserialize, Debug, Eq, PartialEq, Clone)]
-pub struct InstallRuntime {
-    /// Experimental: Use a linked runtime instead of downloading from Mozilla.
+#[derive(Deserialize, Debug, Eq, PartialEq, Clone, Default)]
+pub struct InstallRuntimeOptions {
+    /// Whether to use a linked runtime instead of downloading from Mozilla (experimental, default: `false`).
+    #[serde(default)]
     pub link: bool,
 }
 
-#[cfg(not(feature = "linked-runtime"))]
-#[derive(Debug, Eq, PartialEq, Clone)]
-pub struct InstallRuntime;
+pub type InstallRuntime = Option<InstallRuntimeOptions>;
 
 /// Uninstalls the Firefox runtime.
 ///
@@ -175,8 +172,10 @@ pub struct InstallRuntime;
 ///
 /// [`ConnectorResponse::RuntimeUninstalled`] - No Data
 ///
-#[derive(Debug, Eq, PartialEq, Clone)]
-pub struct UninstallRuntime;
+#[derive(Deserialize, Debug, Eq, PartialEq, Clone, Default)]
+pub struct UninstallRuntimeOptions {}
+
+pub type UninstallRuntime = Option<UninstallRuntimeOptions>;
 
 /// Gets all installed web apps.
 ///
@@ -377,11 +376,11 @@ pub struct UpdateSite {
     /// If empty, no handlers are registered to the operating system.
     pub enabled_protocol_handlers: Option<Vec<String>>,
 
-    /// Whether the web app should be launched on the system login.
+    /// Whether the web app should be launched on the system login (default: `false`).
     #[serde(default)]
     pub launch_on_login: Option<bool>,
 
-    /// Whether the web app should be launched on the browser launch.
+    /// Whether the web app should be launched on the browser launch (default: `false`).
     #[serde(default)]
     pub launch_on_browser: Option<bool>,
 
@@ -402,7 +401,7 @@ pub struct UpdateSite {
 ///
 /// # Parameters
 ///
-/// None.
+/// See [fields](#fields).
 ///
 /// # Returns
 ///
@@ -515,7 +514,7 @@ pub struct UpdateProfile {
 ///
 /// # Parameters
 ///
-/// None.
+/// See [fields](#fields).
 ///
 /// # Returns
 ///
@@ -583,6 +582,9 @@ pub struct UnregisterProtocolHandler {
 /// Contains a HTTP client configuration.
 #[derive(Deserialize, Debug, Eq, PartialEq, Clone, Default)]
 pub struct HTTPClientConfig {
+    /// A custom user-agent header.
+    pub user_agent: Option<String>,
+
     /// A list of paths to DER certificate files.
     pub tls_root_certificates_der: Option<Vec<PathBuf>>,
 
@@ -602,6 +604,7 @@ pub struct HTTPClientConfig {
 impl Into<crate::console::app::HTTPClientConfig> for HTTPClientConfig {
     fn into(self) -> crate::console::app::HTTPClientConfig {
         crate::console::app::HTTPClientConfig {
+            user_agent: self.user_agent,
             tls_root_certificates_der: self.tls_root_certificates_der,
             tls_root_certificates_pem: self.tls_root_certificates_pem,
             tls_danger_accept_invalid_certs: self.tls_danger_accept_invalid_certs,
@@ -612,9 +615,6 @@ impl Into<crate::console::app::HTTPClientConfig> for HTTPClientConfig {
 
 deserialize_unit_struct!(GetSystemVersions);
 deserialize_unit_struct!(GetConfig);
-#[cfg(not(feature = "linked-runtime"))]
-deserialize_unit_struct!(InstallRuntime);
-deserialize_unit_struct!(UninstallRuntime);
 deserialize_unit_struct!(GetSiteList);
 deserialize_unit_struct!(GetProfileList);
 
