@@ -1,15 +1,14 @@
 use std::cmp::Ordering;
 use std::convert::TryInto;
-use std::fs::{create_dir_all, remove_dir_all, rename, write, File, Permissions};
+use std::fs::{File, create_dir_all, remove_dir_all, rename, write};
 use std::io::{BufWriter, Read, Write};
-use std::os::unix::fs::PermissionsExt;
 use std::path::Path;
 use std::process::{Child, Command, Stdio};
 
-use anyhow::{bail, Context, Result};
+use anyhow::{Context, Result, bail};
 use icns::{IconFamily, IconType, Image, PixelFormat};
+use image::imageops::FilterType::Lanczos3;
 use image::imageops::resize;
-use image::imageops::FilterType::Gaussian;
 use image::{DynamicImage, Rgba, RgbaImage};
 use log::{debug, error, warn};
 use reqwest::blocking::Client;
@@ -22,7 +21,7 @@ use crate::components::site::Site;
 use crate::integrations::categories::MACOS_CATEGORIES;
 use crate::integrations::utils::{
     download_icon,
-    generate_icon,
+    generate_fallback_icon,
     normalize_category_name,
     sanitize_name,
 };
@@ -39,11 +38,13 @@ const CREATE_ICON_FILE_ERROR: &str = "Failed to create icon file";
 const CREATE_TEMP_FILE_ERROR: &str = "Failed to create temporary file";
 const CREATE_APPLICATION_DIRECTORY_ERROR: &str = "Failed to create application directory";
 const WRITE_APPLICATION_FILE_ERROR: &str = "Failed to write application file";
+const WRITE_LOADER_SOURCE_ERROR: &str = "Failed to write loader source";
+const COMPILE_LOADER_SOURCE_ERROR: &str = "Failed to compile loader source";
 const STORE_ICONS_ERROR: &str = "Failed to store icons";
 const LAUNCH_APPLICATION_BUNDLE: &str = "Failed to launch web app via system integration";
 const APP_BUNDLE_NAME_ERROR: &str = "Failed to get name of app bundle";
 const APP_BUNDLE_UNICODE_ERROR: &str = "Failed to check name of app bundle for Unicode validity";
-const GENERATE_ICON_ERROR: &str = "Failed to generate icon";
+const GENERATE_FALLBACK_ICON_ERROR: &str = "Failed to generate fallback icon";
 const GET_LETTER_ERROR: &str = "Failed to get first letter";
 
 const ICON_SAFE_ZONE_FACTOR: f64 = 0.697265625;
@@ -79,11 +80,7 @@ impl MacOSIconSize {
     }
 
     fn size(&self) -> u32 {
-        if self.hdpi {
-            self.size * 2
-        } else {
-            self.size
-        }
+        if self.hdpi { self.size * 2 } else { self.size }
     }
 }
 
@@ -130,11 +127,7 @@ fn sort_icons_for_size(icons: &mut [&IconResource], size: &ImageSize) {
         let size1 = size1.unwrap();
         let size2 = size2.unwrap();
 
-        if size1 >= size && size2 >= size {
-            size1.cmp(size2)
-        } else {
-            size1.cmp(size2).reverse()
-        }
+        if size1 >= size && size2 >= size { size1.cmp(size2) } else { size1.cmp(size2).reverse() }
     };
 
     // Compare icons by purpose, and by size if purposes are the same
@@ -182,14 +175,14 @@ fn store_icons(target: &Path, name: &str, icons: &[IconResource], client: &Clien
     for size in &icon_sizes {
         let img_size = size.size();
 
-        debug!("Looking for icon size {}", img_size);
+        debug!("Looking for icon size {img_size}");
         sort_icons_for_size(icons, &ImageSize::Fixed(img_size, img_size));
 
         for icon in &mut *icons {
             // Wrapped into a closure to emulate currently unstable `try` blocks
             let mut process = || -> Result<()> {
                 let url: Url = icon.src.clone().try_into().context(CONVERT_ICON_URL_ERROR)?;
-                debug!("Processing icon {}", url);
+                debug!("Processing icon {url}");
 
                 // Download the image from the URL and load it as RGBA
                 let (bytes, img_type) = download_icon(url, client).context(DOWNLOAD_ICON_ERROR)?;
@@ -205,7 +198,7 @@ fn store_icons(target: &Path, name: &str, icons: &[IconResource], client: &Clien
                     size.icon_type(),
                 )?;
 
-                debug!("Added size {}", img_size);
+                debug!("Added size {img_size}");
                 Ok(())
             };
 
@@ -213,7 +206,7 @@ fn store_icons(target: &Path, name: &str, icons: &[IconResource], client: &Clien
             match process().context(PROCESS_ICON_ERROR) {
                 Ok(_) => break,
                 Err(error) => {
-                    error!("{:?}", error);
+                    error!("{error:?}");
                     warn!("Falling back to the next available icon");
                 }
             }
@@ -228,7 +221,8 @@ fn store_icons(target: &Path, name: &str, icons: &[IconResource], client: &Clien
 
         for size in &icon_sizes {
             let image_size = ImageSize::Fixed(size.size(), size.size());
-            let image_data = generate_icon(letter, &image_size).context(GENERATE_ICON_ERROR)?;
+            let image_data = generate_fallback_icon(letter, &image_size)
+                .context(GENERATE_FALLBACK_ICON_ERROR)?;
 
             let mut img = DynamicImage::ImageRgb8(image_data).into_rgba8();
             mask_icon(&mut img, true).context(MASK_ICON_ERROR)?;
@@ -277,7 +271,7 @@ fn load_icon(content: &[u8], content_type: &str, size: u32) -> Result<RgbaImage>
     // Parse raster icons using the `image` crate and resize it to the correct size
     debug!("Processing as raster icon");
     let img = image::load_from_memory(content).context("Failed to load raster icon")?;
-    let img = img.resize_to_fill(size, size, Gaussian).into_rgba8();
+    let img = img.resize_to_fill(size, size, Lanczos3).into_rgba8();
     Ok(img)
 }
 
@@ -293,8 +287,8 @@ fn mask_icon(icon: &mut RgbaImage, maskable: bool) -> Result<()> {
     let icon_size = Point { x: icon.width(), y: icon.height() };
     let mask = image::load_from_memory(include_bytes!("../../../assets/icon-mask-macos.png"))?;
     let shadow = image::load_from_memory(include_bytes!("../../../assets/icon-shadow-macos.png"))?;
-    let scaled_mask = mask.resize(icon_size.x, icon_size.y, Gaussian); // This is really slow in debug builds, up to ~1s
-    let scaled_shadow = shadow.resize(icon_size.x, icon_size.y, Gaussian); // This is really slow in debug builds, up to ~1s
+    let scaled_mask = mask.resize(icon_size.x, icon_size.y, Lanczos3); // This is really slow in debug builds, up to ~1s
+    let scaled_shadow = shadow.resize(icon_size.x, icon_size.y, Lanczos3); // This is really slow in debug builds, up to ~1s
     let mask_data = scaled_mask.into_rgba8();
     let shadow_data = scaled_shadow.into_rgba8();
 
@@ -315,7 +309,7 @@ fn mask_icon(icon: &mut RgbaImage, maskable: bool) -> Result<()> {
     let scaled_icon_data: RgbaImage = if maskable {
         icon.clone()
     } else {
-        resize(icon, scaled_icon_size.x, scaled_icon_size.y, Gaussian)
+        resize(icon, scaled_icon_size.x, scaled_icon_size.y, Lanczos3)
     };
 
     let background = RgbaImage::from_pixel(icon_size.x, icon_size.y, Rgba([255, 255, 255, 255]));
@@ -372,7 +366,7 @@ fn verify_app_is_pwa(app_bundle: &Path, app_id: &str) -> Result<()> {
 
     let pkg_info_id = format!("APPL{app_id}");
     debug!("Verifying if a bundle is a web app");
-    debug!("'{}' should be '{}'", pkg_info_content, pkg_info_id);
+    debug!("'{pkg_info_content}' should be '{pkg_info_id}'");
 
     if pkg_info_content != pkg_info_id {
         let bundle_name = app_bundle
@@ -482,9 +476,8 @@ fn create_app_bundle(args: &IntegrationInstallArgs) -> Result<()> {
     plist::to_file_xml(info_plist, &info_plist_value).context(WRITE_APPLICATION_FILE_ERROR)?;
     write(pkg_info, format!("APPL{appid}")).context(WRITE_APPLICATION_FILE_ERROR)?;
 
-    // Create and compile loader executable using Swift compiler
-    // Swift compiler (swiftc) is part of Command Line Tools for Xcode which is required by Homebrew
-    // We can assume users will have it installed, but provide old script-based fallback just in case
+    // Create and compile a custom loader executable using the Swift compiler
+    // Swift compiler (swiftc) is part of Xcode Command Line Tools, which are required by Homebrew
     if Command::new("xcode-select").stdout(Stdio::null()).arg("-p").status().is_ok() {
         let loader_source_content = format!(
             r#"import Foundation
@@ -505,27 +498,23 @@ task.waitUntilExit()
         loader_source_file
             .as_file_mut()
             .write_all(loader_source_content.as_bytes())
-            .context("Failed to write loader source")?;
+            .context(WRITE_LOADER_SOURCE_ERROR)?;
 
-        Command::new("swiftc")
+        debug!("Compiling loader source");
+
+        let status = Command::new("swiftc")
             .arg("-O")
             .arg("-o")
             .arg(loader)
             .arg(loader_source_file.path().as_os_str())
             .status()
-            .context("Failed to compile loader source")?;
-    } else {
-        warn!("Could not find Command Line Tools for Xcode");
-        warn!("Falling back to the legacy script-based loader");
-        warn!("Tools can be installed using: xcode-select --install");
-        warn!("After installing, update your web app to apply changes");
+            .context(COMPILE_LOADER_SOURCE_ERROR)?;
 
-        #[rustfmt::skip]
-        let loader_content = format!("#!/usr/bin/env sh\n\n{exe} site launch --direct-launch {ulid} \"$@\"\n");
-        let mut loader_file = File::create(loader).context(WRITE_APPLICATION_FILE_ERROR)?;
-        let loader_permissions = Permissions::from_mode(0o755);
-        loader_file.set_permissions(loader_permissions).context(WRITE_APPLICATION_FILE_ERROR)?;
-        loader_file.write_all(loader_content.as_ref()).context(WRITE_APPLICATION_FILE_ERROR)?;
+        if !status.success() {
+            bail!(COMPILE_LOADER_SOURCE_ERROR);
+        }
+    } else {
+        bail!("Could not find Xcode Command Line Tools");
     }
 
     // Update icons if needed
